<?php
/***************************************************************
*  Copyright notice
*
*  (c) 2005 Morten Tranberg Hansen (mth@daimi.au.dk)
*  All rights reserved
*
*  This script is part of the TYPO3 project. The TYPO3 project is
*  free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  The GNU General Public License can be found at
*  http://www.gnu.org/copyleft/gpl.html.
*
*  This script is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  This copyright notice MUST APPEAR in all copies of the script!
***************************************************************/
/**
 * Plugin 'Events listing' for the 'daimi_event' extension.
 *
 * @author	Morten Tranberg Hansen <mth@daimi.au.dk>
 */

require_once(t3lib_extmgm::extPath('daimi_event').'class.tx_daimievent_picommon.php');
require_once(t3lib_extMgm::extPath('mth_feedit').'class.tx_mthfeedit.php');

class tx_daimievent_pi2 extends tx_daimievent_picommon {
  var $prefixId = 'tx_daimievent_pi2';		// Same as class name
  var $scriptRelPath = 'pi2/class.tx_daimievent_pi2.php';	// Path to this script relative to the extension dir.
  var $extKey = 'daimi_event';	// The extension key.
  var $pi_checkCHash = TRUE;
  
  var $eventUid;
  var $mode;
  var $mthfeedit;

  /**
   * [Put your description here]
   */
  function init($content,$conf)	{
    $this->conf=$conf;		// Setting the TypoScript passed to this function in $this->conf
    $this->pi_setPiVarDefaults();
    $this->pi_loadLL();		// Loading the LOCAL_LANG values    
    $this->pi_initPIflexForm();   // Init FlexForm configuration for plugin

    // Load TCA
    $GLOBALS['TSFE']->includeTCA();
    t3lib_div::loadTCA('tx_daimievent_participants');
    
    $this->eventUid = intval($this->piVars['uid']);

    // Check for event to register for
    if(!$this->eventUid)
      return 'No event uid given.';

    // Get event record
    $this->internal['currentTable'] = 'tx_daimievent_events';
    $this->internal['currentRow'] = $this->pi_getRecord('tx_daimievent_events',$this->eventUid);
    if(empty($this->internal['currentRow']))
      return 'No event with given uid ('.$this->eventUid.')';

    // Check owner of event
    $partner = $this->pi_getFFvalue($this->cObj->data['pi_flexform'], 'field_partner', 'sDEF');
    $this->conf['partner'] = $partner ? intval($partner) : intval($this->conf['partner']);
    $owner = $this->getOwnerOfCurrentRow();
    if($this->conf['partner']!=$owner['uid'])
      return 'The event ('.$this->eventUid.') can not be registered at this partner (note to admin: there might be multible partners with same storage folder for events).'; 

    // Get registration mode; 1: Registration possible; 0: Registration not possible; -1: Waitinglist possible
    $this->mode = $this->getRegistrationMode();  

    // Set default values
    $participants = $this->pi_getFFvalue($this->cObj->data['pi_flexform'], 'field_participants', 'sDEF');
    $this->conf['mthfeedit.']['pid'] = $participants ? $participants : $this->conf['mthfeedit.']['pid'];
    $this->conf['mthfeedit.']['create.']['overrideValues.']['event'] = $this->eventUid;
    $this->conf['mthfeedit.']['create.']['overrideValues.']['status'] = $this->mode==1?'registered':'waitinglist';
    $this->conf['mthfeedit.']['create.']['overrideValues.']['registration_time'] = time();
    $this->conf['mthfeedit.']['text_in_top_of_form'] = $this->mode==-1?$this->getFieldContent('registration_info_no_slots'):$this->getFieldContent('registration_info_free_slots');
    $this->conf['mthfeedit.']['text_in_top_of_preview'] = $this->conf['mthfeedit.']['text_in_top_of_form'];
    $this->LOCAL_LANG['default']['create_saved_message'] = ($this->mode==-1?$this->getFieldContent('registration_completed_no_slots'):$this->getFieldContent('registration_completed_free_slots')) . '<p'.$this->pi_classParam('email-notice').'>' . $this->pi_getLL('email_has_been_sent') .'</p>';

    $this->conf['mthfeedit.']['email.']['from'] = $this->conf['email.']['fromEmail'] ? $this->conf['email.']['fromEmail'] : $this->conf['mthfeedit.']['email.']['from'];
    $this->conf['mthfeedit.']['email.']['fromName'] = $this->conf['email.']['fromName'] ? $this->conf['email.']['fromName'] : $this->conf['mthfeedit.']['email.']['fromName'];


    // Allow edit in all fields in participants records
    t3lib_div::loadTCA('tx_daimievent_participants');
    $this->conf['mthfeedit.']['create.']['fields'] = implode(',',array_keys($GLOBALS['TCA']['tx_daimievent_participants']['columns']));

    // Get event
    $event = $this->pi_getRecord('tx_daimievent_events',$this->eventUid);
    $this->internal['currentRow'] = $event;
    $this->internal['currentTable'] = 'tx_daimievent_events';

    // Get show_fields according to var from extionsion manager and excluded fields from event
    $excluded_fields = t3lib_div::trimExplode(',',$event['excluded_reg_fields']);
    if(!$this->getFieldContent('pricing'))
      $excluded_fields[] = 'price';
    if(!$this->getFieldContent('registration_types'))
      $excluded_fields[] = 'registration_type';
    $show_fields = t3lib_div::trimExplode(',',$GLOBALS['TYPO3_CONF_VARS']['EXTCONF'][$this->extKey]['possibleRegistrationFields']);
    $this->conf['mthfeedit.']['create.']['show_fields'] = implode(',',array_diff($show_fields,$excluded_fields));

    // Add a required label to fields defined as required in event.
    $required_fields = array_diff(t3lib_div::trimExplode(',',$event['required_reg_fields']),$excluded_fields);
    if($this->getFieldContent('pricing'))
      $required_fields[] = 'price';
    if($this->getFieldContent('registration_types'))
      $required_fields[] = 'registration_type';
    foreach($required_fields as $field) {
      $evalValues = t3lib_div::trimExplode(',',$this->conf['mthfeedit.']['create.']['evalValues.'][$field]);
      if(!in_array('required',$evalValues)) {
	$this->conf['mthfeedit.']['create.']['evalValues.'][$field] = $this->conf['mthfeedit.']['create.']['evalValues.'][$field]?',required':'required';
      }
    }

    // Set TCA so only registration types selected in event is visible in registration form
    $GLOBALS['TCA']['tx_daimievent_participants']['columns']['registration_type']['config']['foreign_table_where'] = 
      'AND tx_daimievent_registration_types.uid IN ('.($event['registration_types']?$event['registration_types']:0).') '.
      $GLOBALS['TCA']['tx_daimievent_participants']['columns']['registration_type']['config']['foreign_table_where'];
    
       
    // add pi2 style sheet to page
    $GLOBALS['TSFE']->additionalHeaderData[] = '<link rel="stylesheet" type="text/css" href="'.t3lib_extMgm::siteRelPath($this->extKey).'res/style_pi2.css" />';
  }
  
  /**
   * [Put your description here]
   */
  function main($content,$conf)	{
    $initError = $this->init($content,$conf);
    if(!empty($initError))
      return $this->pi_wrapInBaseClass('ERROR: '.$initError);

#    debug($this->conf);

    // Make header of registration form.
    // FIXME: Issue #765 mangler at der kommer class-attribut på h1-tag'et

    $content .= '
	        <h1 class="'.$this->pi_getClassName('registration-header').'">'.$this->pi_getLL('registration').'</h1>      
	        <div'.$this->pi_classParam('title').'>'.($this->getFieldContent('category')?$this->getFieldContent('category').': ':'').$this->getFieldContent('title').'</div>      
	        <div'.$this->pi_classParam('timeandplace').'>'.$this->getFieldContent('start_date').($this->getFieldContent('end_date')?' - '.$this->getFieldContent('end_date'):'').': '.$this->getFieldContent('time').($this->getFieldContent('address')?nl2br($this->getFieldContent('address')):'').($this->getFieldContent('postal_code') && $this->getFieldContent('city')?'<br />'.$this->getFieldContent('postal_code').' '.$this->getFieldContent('city'):'').'</div>
    ';

    // Generate content of registration form
   
    if($this->mode==0) {                                                 // Registration NOT possible
      $content .= '<p>'.$this->pi_getLL('registration_not_possible').'</p>';
    } elseif($this->internal['currentRow']['registration_deadline'] < time()) {
      $content .= '<p>'.$this->pi_getLL('registration_deadline_passed').'</p>';
    } else {                                                             // Registration possible => generate form
      
      // Generate new TCA settings for prices.

      $oldPriceConfig = $GLOBALS['TCA']['tx_daimievent_participants']['columns']['price']['config'];      
      $prices = $this->getFieldsArrayFromFF($this->internal['currentRow']['pricing'], 'pricing');

      $priceConfig = array('type' => 'select',
			   'items' => array(array(''=>'')));
      foreach($prices as $price) {
	$priceCat = $this->pi_getRecord('tx_daimievent_price_categories',$price['category']);
	$priceConfig['items'][] = array($priceCat['title'].': '.($price['price']==0 ? $this->pi_getLL('picommon.free') : $price['price'].' '.$this->pi_getLL('picommon.currency')),$this->encodePrice($price)); // we are encodeing the price as an integer, so both pricecategory and price is saved. This is decoded again in the afterSave function.
      }

      $GLOBALS['TCA']['tx_daimievent_participants']['columns']['price']['config'] = $priceConfig;

      // Call mth_feedit
      $this->mthfeedit = t3lib_div::makeInstance('tx_mthfeedit');
      $content .= $this->mthfeedit->init($this,$this->conf['mthfeedit.']);
      
#      debug($this->mthfeedit,'affected');

      // Clean up.
      $GLOBALS['TCA']['tx_daimievent_participants']['columns']['price']['config'] = $oldPriceConfig;

      $content .= '<div'.$this->pi_classParam("back-link").'>'.$this->pi_linkTP($this->pi_getLL('back','Back'),array('tx_daimievent_pi1' => array('singlePid'=>'', 'listPid' => $this->piVars['listPid'],'showUid' => $this->eventUid)),1,$this->piVars['singlePid']).'</div>';
    }

    return $this->pi_wrapInBaseClass($content);
  }


  function getEmailTemplate() {
    $fields = $this->mthfeedit->makeTEXTPreview('all');

    return '
<!-- ###EMAIL_TEMPLATE_CREATE_SAVED### begin -->
'.$this->pi_getLL('email.title').'

<!--###SUB_RECORD###-->
'.strip_tags($this->mode==-1?$this->getFieldContent('registration_completed_no_slots'):$this->getFieldContent('registration_completed_free_slots')).'

'.$this->pi_getLL('email.your_informations').'

'.$fields.'
'.$this->pi_getLL('email.admin_contact').' '.$this->conf['email.']['admin'].'

<!--###SUB_RECORD###-->
<!-- ###EMAIL_TEMPLATE_CREATE_SAVED### end-->
';
  }


  function getEmailField($field) {
    switch((string)$field) {
    case 'price':
      $price = $this->decodePrice();
      break;
    default:
      return '###FIELD_'.$field.'###';
      break;
    }
  }


 function afterSave($content,$conf) {
    $mthfeedit = t3lib_div::makeInstance('tx_mthfeedit');
    $mthfeedit->afterSave($content, $conf);
   
    // Convert price witch are stored encoded with price categories and price to only contain the price.
    $price = $this->decodePrice($content['rec']['price']);
    $GLOBALS['TYPO3_DB']->exec_UPDATEquery('tx_daimievent_participants','uid='.$content['rec']['uid'],array('price' => $price['price']));

 }


  function encodePrice($price) {
    return $this->dec2string($price['category'],8).'9'.$this->dec2string($price['price'],8);
  }

  
  function decodePrice($string) {
    $price = array();
    $codeArr = t3lib_div::trimExplode('9',$string,1);    
    if(count($codeArr)==1)
      return $string;

    $price['category'] = $this->string2dec($codeArr[0],8);
    $price['price'] = $this->string2dec($codeArr[1],8);

    return $price;
  }
       

  function string2dec ($string, $base) { 
    $decimal = 0; 
    $base = (int)$base; 
    $string = trim($string);

    if ($base < 2 | $base > 36 | $base == 10 || empty($string)) { 
      return 0;
    } // if; 
    
    $charset = '0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ'; 
    $charset = substr($charset, 0, $base); 
      
    do {     
      $char   = substr($string, 0, 1); 
      $string = substr($string, 1); 

      $pos = strpos($charset, $char); 
      if ($pos === false) { 
	return 0; 
      } // if 

      $decimal = ($decimal * $base) + $pos;      

    } while($string <> null); 
    
    return $decimal;    
  }
  
  
  function dec2string ($decimal, $base) {
    $string = null; 
    
    $base = (int)$base; 
    if ($base < 2 | $base > 36 | $base == 10 || !ereg('(^[0-9]{1,16}$)', trim($decimal))) { 
      return '';
    } // if; 
    
    $charset = '0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ'; 
    $charset = substr($charset, 0, $base); 
    
    $decimal = (int)$decimal;  
    
    do {
      $remainder = ($decimal % $base); 
    
      $char   = substr($charset, $remainder, 1); 
      $string = "$char$string"; 
      
      $decimal   = ($decimal - $remainder) / $base; 
      
    } while ($decimal > 0); 
    
    return $string; 
    
  }

  
  
}



if (defined('TYPO3_MODE') && $TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/daimi_event/pi2/class.tx_daimievent_pi2.php'])	{
  include_once($TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/daimi_event/pi2/class.tx_daimievent_pi2.php']);
}

?>